@@ -4,7 +4,7 @@
     A utility to process facsimile received with the ELSA
     MicroLink(tm) Office modem.
 
-    Copyright (C) 1999-2000 Oliver Gantz <o.gantz@tu-bs.de>
+    Copyright (C) 1999-2000 Oliver Gantz <Oliver.Gantz@epost.de>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,17 +24,10 @@
     ELSA and MicroLink are trademarks of ELSA AG, Aachen.
 */
 
-#include "mlofile.h"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif 
-
 #include <stdio.h>
 #include <string.h>
 
-#include <qfile.h>
-
+#include "mlofile.h"
 #include "global.h"
 
 
@@ -170,9 +163,11 @@ static const hcode_t mode_codes[] = {
 
 
 
-MLOFile::MLOFile( const char * name ) : QFile( name )
+MLOFile::MLOFile(const char *name) : QFile()
 {
 	init();
+
+	setName(name);
 }
 
 
@@ -187,9 +182,18 @@ MLOFile::~MLOFile()
 }
 
 
+void MLOFile::setName(const char *name)
+{
+	if (name[0] == '/')
+		QFile::setName(name);
+	else
+		QFile::setName(expandPath(name));
+}
+
+
 bool MLOFile::open()
 {
-	return QFile::open( IO_ReadOnly );
+	return QFile::open(IO_ReadOnly);
 }
 
 
@@ -200,7 +204,7 @@ void MLOFile::close()
 }
 
 
-int MLOFile::readLine( char * line, uint maxlen )
+int MLOFile::readLine(char *line, uint maxlen)
 {
 	uint i = 0;
 	int c;
@@ -227,55 +231,55 @@ int MLOFile::readLine( char * line, uint maxlen )
 	line[i] = 0;
 
 	if (!strncmp(line, "CONNECT", 7))
-		data_mode = TRUE;
+		data_mode = true;
 
 	return i;
 }
 
 
-void MLOFile::readRleLine( short line[] )
+void MLOFile::readRleLine(short line[])
 {
-	decodeDataLine( line, FALSE );
+	decodeDataLine(line, false);
 }
 
 
-void MLOFile::readImgLine( uchar line[] )
+void MLOFile::readImgLine(uchar line[], bool lsb_first)
 {
 	short rle_buf[MAX_RLE_BUFF], pos = 0;
 	int count = 0;
 
-	readRleLine( rle_buf );
+	readRleLine(rle_buf);
 
-	memset( (void *)line, 0x00, ((page_width-1) >> 3) + 1);
+	memset((void *)line, 0x00, ((page_width-1) >> 3) + 1);
 
 	while ((pos < page_width) && (rle_buf[count] != -1)) {
 		pos += rle_buf[count++];
 		if ((pos >= page_width) || (rle_buf[count] == -1))
 			break;
-		fillDataLine( line, pos, rle_buf[count] );
+		fillDataLine(line, pos, rle_buf[count], lsb_first);
 		pos += rle_buf[count++];
 	}
 }
 
 
-int MLOFile::readHuffLine( uchar line[] )
+int MLOFile::readHuffLine(uchar line[])
 {
 	short rle_buf[MAX_RLE_BUFF];
 
-	readRleLine( rle_buf );
+	readRleLine(rle_buf);
 
-	return encodeDataLine( line );
+	return encodeDataLine(line);
 }
 	
 
-const char * MLOFile::sender()
+const char *MLOFile::sender()
 {
 	char buff[80];
 
 	if (fax_sender[0])
 		return fax_sender;
 
-	memset( fax_sender, ' ', 20 );
+	memset(fax_sender, ' ', 20);
 	fax_sender[20] = 0;
 
 	at(0);
@@ -284,7 +288,7 @@ const char * MLOFile::sender()
 	while (!atEnd()) {
 		readLine(buff, 80);
 		if (!strncmp(buff, "+FTI:", 5)) {
-			char * p, * q;
+			char *p, *q;
 
 			if (!(p = strchr(buff, '\"')))
 				return fax_sender;
@@ -322,18 +326,19 @@ int MLOFile::pages()
 }
 
 
-bool MLOFile::gotoPage( int page )
+bool MLOFile::gotoPage(int page)
 {
 	char buff[80];
 
 	if (fax_page != page) {
+		page_params = 0;
 		page_width = 1728;
 		page_height = 0;
 		page_used_height = 0;
 		page_length = PAGE_LEN_A4;
 		page_lpi = 98;
 		page_bit_rate = 2400;
-		page_2d = FALSE;
+		page_2d = false;
 	}
 
 	init_data();
@@ -342,12 +347,13 @@ bool MLOFile::gotoPage( int page )
 	while (!atEnd()) {
 		readLine(buff, 80);
 		if (!strncmp(buff, "+FCS:", 5)) {
-			int vr = 0, br = 0, wd = 0, ln = 0, df = 0;
+			int vr = 0, br = 0, wd = 0, ln = 0, df = 0, ec = 0, bf = 0, st = 0;
 			static int lpis[2] = { 98, 196 };
 			static int ratings[6] = { 2400, 4800, 7200, 9600, 12000, 14400 };
 			static int widths[5] = { 1728, 2048, 2432, 1216, 864 };
 
-			sscanf(buff, "+FCS:%d,%d,%d,%d,%d", &vr, &br, &wd, &ln, &df);
+			sscanf(buff, "+FCS:%d,%d,%d,%d,%d,%d,%d,%d", &vr, &br, &wd, &ln, &df, &ec, &bf, &st);
+			page_params = (vr&1)|((br&7)<<1)|((wd&7)<<4)|((ln&3)<<7)|((df&3)<<9)|((ec&1)<<11)|((bf&1)<<12)|((st&7)<<13);
 			if ((vr >= 0) && (vr < 2))
 				page_lpi = lpis[vr];
 			if ((br >= 0) && (br < 6))
@@ -362,46 +368,47 @@ bool MLOFile::gotoPage( int page )
 				refline[0] = page_width;
 				refline[1] = -1;
 				fax_page = page;
-				return TRUE;
+				return true;
 			}
 		}
 	}
 
-	return FALSE;
+	return false;
 }
 
 
-bool MLOFile::readPageInfo( int page, page_info_t * info )
+bool MLOFile::readPageInfo(int page, page_info_t *info)
 {
 	if (fax_page != page)
-		if (!gotoPage( page ))
-			return FALSE;
+		if (!gotoPage(page))
+			return false;
 
+	info->params = page_params;
 	info->lpi = page_lpi;
 	info->bit_rate = page_bit_rate;
 	info->width = page_width;
 	info->length = page_length;
 	info->comp_2d = page_2d;
 
-	return TRUE;
+	return true;
 }
 
 
-int MLOFile::pageHeight( int page )
+int MLOFile::pageHeight(int page)
 {
 	int eols;
 	
 	if ((fax_page == page) && page_height)
 		return page_height;
 
-	if (!gotoPage( page ))
+	if (!gotoPage(page))
 		return 0;
 
 	while (skipEol()) {
 		eols = 1;
 		if (!getDataBits(12)) {
 #ifdef MLOFILE_DEBUG
-			fprintf(stderr, "MLOFile: Unexpected end of file in pageHeight( %d ).\n", page);
+			fprintf(stderr, "MLOFile: Unexpected end of file in pageHeight(%d).\n", page);
 #endif
 			return page_height;
 		}
@@ -409,7 +416,7 @@ int MLOFile::pageHeight( int page )
 			cutDataBits(12);
 			if ((!getDataBits(12)) || (eols++ == 6))
 				return page_height;
-		} 
+		}
 		page_height++;
 	}
 
@@ -417,7 +424,7 @@ int MLOFile::pageHeight( int page )
 }
 
 
-int MLOFile::pageUsedHeight( int page )
+int MLOFile::pageUsedHeight(int page)
 {
 	int lines;
 	short rle_buf[MAX_RLE_BUFF];
@@ -425,16 +432,16 @@ int MLOFile::pageUsedHeight( int page )
 	if ((fax_page == page) && page_used_height)
 		return page_used_height;
 
-	if (!(lines = pageHeight( page )))
+	if (!(lines = pageHeight(page)))
 		return 0;
 
-	if (!gotoPage( page ))
+	if (!gotoPage(page))
 		return 0;
 
 	page_height = lines;
 
 	for (lines=0; lines < page_height; lines++) {
-		decodeDataLine( rle_buf, TRUE );
+		decodeDataLine(rle_buf, true);
 		if ((rle_buf[0] != -1) && (rle_buf[1] != -1))
 			page_used_height = lines + 1;
 	}
@@ -452,13 +459,14 @@ void MLOFile::init()
 
 	fax_sender[0] = 0;
 
+	page_params = 0;
 	page_width = 0;
 	page_height = 0;
 	page_used_height = 0;
 	page_length = 0;
 	page_lpi = 0;
 	page_bit_rate = 0;
-	page_2d = FALSE;
+	page_2d = false;
 
 	refline[2432] = -1;
 }
@@ -466,7 +474,7 @@ void MLOFile::init()
 
 void MLOFile::init_data()
 {
-	data_mode = FALSE;
+	data_mode = false;
 
 	data_buff = 0;
 	data_width = 0;
@@ -481,52 +489,52 @@ int MLOFile::getDataCh()
 		if ((c = getch()) == CDLE)
 			return c;
 		if ((c == CETX) || (c == -1)) {
-			data_mode = FALSE;
+			data_mode = false;
 			return -1;
 		}
 	}
 
 	if (c == -1)
-		data_mode = FALSE;
+		data_mode = false;
 
 	return c;
 }
 	
 
-bool MLOFile::getDataBits( uchar len )
+bool MLOFile::getDataBits(uchar len)
 {
 	int c;
 
 	while (data_width < len) {
 		if ((c = getDataCh()) == -1)
-			return FALSE;
+			return false;
 		data_buff = (data_buff << 8) | (ulong)xchg_endian[c];
 		data_width += 8;
 	}
 
-	return TRUE;
+	return true;
 }
 
 
-void MLOFile::cutDataBits( uchar width )
+void MLOFile::cutDataBits(uchar width)
 {
 	data_width -= width;
 	data_buff &= ((1 << data_width) - 1);
 }
 
 
-bool MLOFile::codeMatches( const uchar code, const uchar width )
+bool MLOFile::codeMatches(const uchar code, const uchar width)
 {
 	return (ushort)(data_buff >> (data_width - width)) == (ushort)code;
 }
 
 
-short MLOFile::findMatch( const hcode_t table[] )
+short MLOFile::findMatch(const hcode_t table[])
 {
 	int i;
 
 	for (i=0; table[i].width; i++)
-		if (codeMatches( table[i].code, table[i].width )) {
+		if (codeMatches(table[i].code, table[i].width)) {
 			cutDataBits(table[i].width);
 			return table[i].len;
 		}
@@ -538,7 +546,7 @@ short MLOFile::findMatch( const hcode_t table[] )
 bool MLOFile::skipEol()
 {
 	if (!getDataBits(12))
-		return FALSE;
+		return false;
 
 	while (data_mode && (!codeMatches(0x01, 12))) {
 		if ((data_buff >> (data_width - 11)) & 0x01)
@@ -546,7 +554,7 @@ bool MLOFile::skipEol()
 		else
 			cutDataBits(1);
 		if (!getDataBits(12))
-			return FALSE;
+			return false;
 	}
 
 	if (data_mode)
@@ -556,7 +564,7 @@ bool MLOFile::skipEol()
 }
 
 
-void MLOFile::fillDataLine( uchar * line, short pos, short len )
+void MLOFile::fillDataLine(uchar *line, short pos, short len, bool lsb_first)
 {
 	short end, a, b;
 	uchar a_mask, b_mask;
@@ -569,8 +577,14 @@ void MLOFile::fillDataLine( uchar * line, short pos, short len )
 	a = pos >> 3;
 	b = end >> 3;
 
-	a_mask = (uchar)(0xff <<  (pos & 0x07));
-	b_mask = (uchar)(0xff >> ((end & 0x07) ^ 0x07));
+	if (lsb_first) {
+		a_mask = (uchar)(0xff <<  (pos & 0x07));
+		b_mask = (uchar)(0xff >> ((end & 0x07) ^ 0x07));
+	}
+	else {
+		a_mask = (uchar)(0xff >>  (pos & 0x07));
+		b_mask = (uchar)(0xff << ((end & 0x07) ^ 0x07));
+	}
 
 	if (a == b)
 		line[a] |= (a_mask & b_mask);
@@ -583,18 +597,18 @@ void MLOFile::fillDataLine( uchar * line, short pos, short len )
 }
 
 
-short MLOFile::getRunLength( const hcode_t table[], uchar max_width )
+short MLOFile::getRunLength(const hcode_t table[], uchar max_width)
 {
 	short len1, len2;
 
-	if (!getDataBits( max_width )) {
+	if (!getDataBits(max_width)) {
 #ifdef MLOFILE_DEBUG
 		fprintf(stderr, "MLOFile: Could not get bits for make-up/terminating code.\n");
 #endif
 		return -1;
 	}
 
-	if ((len1 = findMatch( table )) < 64) {
+	if ((len1 = findMatch(table)) < 64) {
 #ifdef MLOFILE_DEBUG
 		if (len1 < 0)
 			fprintf(stderr, "MLOFile: Could not find make-up/terminating code.\n");
@@ -602,14 +616,14 @@ short MLOFile::getRunLength( const hcode_t table[], uchar max_width )
 		return len1;
 	}
 
-	if (!getDataBits( max_width )) {
+	if (!getDataBits(max_width)) {
 #ifdef MLOFILE_DEBUG
 		fprintf(stderr, "MLOFile: Could not get bits for terminating code.\n");
 #endif
 		return -1;
 	}
 
-	len2 = findMatch( table );
+	len2 = findMatch(table);
 
 	if ((len2 == -1) || (len2 > 63)) {
 #ifdef MLOFILE_DEBUG
@@ -622,22 +636,22 @@ short MLOFile::getRunLength( const hcode_t table[], uchar max_width )
 }
 
 
-bool MLOFile::decode1DDataLine( short run_lengths[], bool probeonly )
+bool MLOFile::decode1DDataLine(short run_lengths[], bool probeonly)
 {
 	short pos = 0;
 	int count = 0;
-	bool retval = TRUE;
+	bool retval = true;
 
 	while ((pos < page_width) && (count < MAX_RLE_BUFF)) {
-		if ((run_lengths[count] = getRunLength( white_codes, 12 )) == -1) {
-			retval = FALSE;
+		if ((run_lengths[count] = getRunLength(white_codes, 12)) == -1) {
+			retval = false;
 			break;
 		}
 		pos += run_lengths[count++];
 		if ((pos >= page_width) || (count >= MAX_RLE_BUFF))
 			break;
-		if ((run_lengths[count] = getRunLength( black_codes, 13 )) == -1) {
-			retval = FALSE;
+		if ((run_lengths[count] = getRunLength(black_codes, 13)) == -1) {
+			retval = false;
 			break;
 		}
 		if (pos + run_lengths[count] > page_width)
@@ -658,7 +672,7 @@ bool MLOFile::decode1DDataLine( short run_lengths[], bool probeonly )
 }
 
 
-bool MLOFile::decode2DDataLine( short run_lengths[], bool probeonly )
+bool MLOFile::decode2DDataLine(short run_lengths[], bool probeonly)
 {
 	short mode, a0 = 0, b1 = 0;
 	int count = 0, ref_count = 0, len;
@@ -668,9 +682,9 @@ bool MLOFile::decode2DDataLine( short run_lengths[], bool probeonly )
 	while ((a0 < page_width) && (count < MAX_RLE_BUFF)) {
 		if (!getDataBits(7)) {
 			run_lengths[count] = -1;
-			return FALSE;
+			return false;
 		}
-		mode = findMatch( mode_codes );
+		mode = findMatch(mode_codes);
 
 		while (((b1 <= a0) && (b1 < page_width)) || ((count & 1) == (ref_count & 1)))
 			b1 += refline[ref_count++];
@@ -683,16 +697,16 @@ bool MLOFile::decode2DDataLine( short run_lengths[], bool probeonly )
 				break;
 			case MODE_H:
 				if (count & 1) {
-					len = getRunLength( black_codes, 13 );
+					len = getRunLength(black_codes, 13);
 					run_lengths[count++] += len;
 					a0 += len;
-					len = getRunLength( white_codes, 12 );
+					len = getRunLength(white_codes, 12);
 				}
 				else {
-					len = getRunLength( white_codes, 12 );
+					len = getRunLength(white_codes, 12);
 					run_lengths[count++] += len;
 					a0 += len;
-					len = getRunLength( black_codes, 13 );
+					len = getRunLength(black_codes, 13);
 				}
 				run_lengths[count++] += len;
 				a0 += len;
@@ -730,7 +744,7 @@ bool MLOFile::decode2DDataLine( short run_lengths[], bool probeonly )
 				break;
 			default:
 				run_lengths[count] = -1;
-				return FALSE;
+				return false;
 		}
 
 		if (probeonly && (count > 1))
@@ -744,13 +758,13 @@ bool MLOFile::decode2DDataLine( short run_lengths[], bool probeonly )
 
 	run_lengths[count] = -1;
 
-	return TRUE;
+	return true;
 }
 
 
-void MLOFile::decodeDataLine( short line[], bool probeonly )
+void MLOFile::decodeDataLine(short line[], bool probeonly)
 {
-	bool one_dim = TRUE, retval;
+	bool one_dim = true, retval;
 
 	if (!skipEol()) {
 		memcpy(line, refline, sizeof(refline));
@@ -767,9 +781,9 @@ void MLOFile::decodeDataLine( short line[], bool probeonly )
 	}
 
 	if (one_dim)
-		retval = decode1DDataLine( line, probeonly );
+		retval = decode1DDataLine(line, probeonly);
 	else
-		retval = decode2DDataLine( line, probeonly );
+		retval = decode2DDataLine(line, probeonly);
 	
 	if (retval)
 		memcpy(refline, line, sizeof(refline));
@@ -778,7 +792,7 @@ void MLOFile::decodeDataLine( short line[], bool probeonly )
 }
 
 
-int MLOFile::storeDataCode( uchar dest[], int pos, const hcode_t & code )
+int MLOFile::storeDataCode(uchar dest[], int pos, const hcode_t& code)
 {
 	ushort data_buff;
 	uchar width, *d;
@@ -813,7 +827,7 @@ int MLOFile::storeDataCode( uchar dest[], int pos, const hcode_t & code )
 }
 
 
-int MLOFile::encodeDataLine( uchar dest[] )
+int MLOFile::encodeDataLine(uchar dest[])
 {
 	static const hcode_t eol = {0x01, 12, 0};
 	static hcode_t fill = {0x00, 0, 0};
@@ -825,13 +839,13 @@ int MLOFile::encodeDataLine( uchar dest[] )
 		
 		if (i & 0x01) {
 			if (length > 63)
-				pos = storeDataCode( dest, pos, black_codes[black_makeup_lut[length >> 6]] );
-			pos = storeDataCode( dest, pos, black_codes[black_term_lut[length & 0x3f]] );
+				pos = storeDataCode(dest, pos, black_codes[black_makeup_lut[length >> 6]]);
+			pos = storeDataCode(dest, pos, black_codes[black_term_lut[length & 0x3f]]);
 		}
 		else {
 			if (length > 63)
-				pos = storeDataCode( dest, pos, white_codes[white_makeup_lut[length >> 6]] );
-			pos = storeDataCode( dest, pos, white_codes[white_term_lut[length & 0x3f]] );
+				pos = storeDataCode(dest, pos, white_codes[white_makeup_lut[length >> 6]]);
+			pos = storeDataCode(dest, pos, white_codes[white_term_lut[length & 0x3f]]);
 		}
 
 		i++;
@@ -840,10 +854,10 @@ int MLOFile::encodeDataLine( uchar dest[] )
 	
 	if ((pos & 0x07) != 0x04) {
 		fill.width = (0x0c - (pos & 0x07)) & 0x07;
-		pos = storeDataCode( dest, pos, fill );
+		pos = storeDataCode(dest, pos, fill);
 	}
 
-	pos = storeDataCode( dest, pos, eol );
+	pos = storeDataCode(dest, pos, eol);
 
 	return pos >> 3;
 }
